package fucoin.configurations;

import akka.actor.ActorRef;
import akka.actor.Props;
import fucoin.AbstractNode;
import fucoin.actions.join.ActionTellSupervisor;
import fucoin.configurations.internal.ConfigurationCreator;
import fucoin.supervisor.SuperVisorImpl;
import fucoin.wallet.WalletImpl;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ThreadLocalRandom;

/**
 *
 */
public abstract class AbstractConfiguration extends AbstractNode {

    protected ActorRef superVisor;

    protected final List<ActorRef> activeActors = new ArrayList<>();

    public static Props props(Class configurationClass) {

        return Props.create(new ConfigurationCreator(configurationClass));
    }

    public ActorRef spawnWallet(String name, boolean createGUI) {
        Props props;
        int numOfWallets = activeActors.size();
        if (numOfWallets == 0) {
            props = WalletImpl.props(null, name, createGUI);
        } else {
            props = WalletImpl.props(activeActors.get(numOfWallets - 1), name, createGUI);
        }

        ActorRef actorRef = context().actorOf(props, name);

        activeActors.add(actorRef);

        if (numOfWallets == 0) {
            //actorRef.tell(new ActionJoinAnswer(cSuperVisorActor), cSuperVisorActor);
            actorRef.tell(new ActionTellSupervisor(superVisor), superVisor);
        }

        return actorRef;
    }

    public void spawnWallets(int n, boolean createGUI) {
        for (int i = 0; i < n; i++) {
            String nameOfTheWallet = "Wallet" + String.valueOf(activeActors.size());
            spawnWallet(nameOfTheWallet, createGUI);
        }
    }

    public ActorRef getRandomWallet() {
        return activeActors.get(ThreadLocalRandom.current().nextInt(activeActors.size()));
    }

    public ActorRef initSupervisor() {
        superVisor = context().actorOf(SuperVisorImpl.props(), "SuperVisorImpl");

        // Don't ask.
        try {
            Thread.sleep(200);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        return superVisor;
    }

    @Override
    public void preStart() throws Exception {
        super.preStart();

        this.run();
    }

    public abstract void run();

    public abstract String getName();

    @Override
    public String toString() {
        return getName();
    }
}
